var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import chai, { expect } from "chai";
import { css, Updates } from "../index.js";
import { html } from "../templating/template.js";
import { uniqueElementName } from "../testing/exports.js";
import { ElementController } from "./element-controller.js";
import { FASTElementDefinition } from "./fast-definitions.js";
import { FASTElement } from "./fast-element.js";
import { HydratableElementController } from "./hydration.js";
import spies from "chai-spies";
chai.use(spies);
describe("The HydratableElementController", () => {
    beforeEach(() => {
        HydratableElementController.install();
    });
    afterEach(() => {
        ElementController.setStrategy(ElementController);
    });
    function createController(config = {}, BaseClass = FASTElement) {
        var _a;
        const name = uniqueElementName();
        const definition = FASTElementDefinition.compose((_a = class ControllerTest extends BaseClass {
            },
            _a.definition = Object.assign(Object.assign({}, config), { name }),
            _a)).define();
        const element = document.createElement(name);
        const controller = ElementController.forCustomElement(element);
        return {
            name,
            element,
            controller,
            definition,
            shadowRoot: element.shadowRoot,
        };
    }
    it("A FASTElement's controller should be an instance of HydratableElementController after invoking 'addHydrationSupport'", () => {
        const { element } = createController();
        expect(element.$fastController).to.be.instanceOf(HydratableElementController);
    });
    describe("without the `defer-hydration` attribute on connection", () => {
        it("should render the element's template", () => __awaiter(void 0, void 0, void 0, function* () {
            var _a;
            const { element } = createController({ template: html `<p>Hello world</p>` });
            document.body.appendChild(element);
            yield Updates.next();
            expect((_a = element.shadowRoot) === null || _a === void 0 ? void 0 : _a.innerHTML).to.be.equal("<p>Hello world</p>");
            document.body.removeChild(element);
        }));
        it("should apply the element's main stylesheet", () => __awaiter(void 0, void 0, void 0, function* () {
            var _b;
            const { element } = createController({ styles: css `:host{ color :red}` });
            document.body.appendChild(element);
            expect((_b = element.$fastController.mainStyles) === null || _b === void 0 ? void 0 : _b.isAttachedTo(element)).to.be.true;
            document.body.removeChild(element);
        }));
        it("should invoke a HostBehavior's connectedCallback", () => __awaiter(void 0, void 0, void 0, function* () {
            const behavior = {
                connectedCallback: chai.spy(() => { })
            };
            const { element, controller } = createController();
            controller.addBehavior(behavior);
            document.body.appendChild(element);
            expect(behavior.connectedCallback).to.have.been.called();
            document.body.removeChild(element);
        }));
    });
    describe("with the `defer-hydration` is set before connection", () => {
        it("should not render the element's template", () => __awaiter(void 0, void 0, void 0, function* () {
            var _a;
            const { element } = createController({ template: html `<p>Hello world</p>` });
            element.setAttribute('defer-hydration', '');
            document.body.appendChild(element);
            yield Updates.next();
            expect((_a = element.shadowRoot) === null || _a === void 0 ? void 0 : _a.innerHTML).be.equal("");
            document.body.removeChild(element);
        }));
        it("should not attach the element's main stylesheet", () => __awaiter(void 0, void 0, void 0, function* () {
            var _b;
            const { element } = createController({ styles: css `:host{ color :red}` });
            element.setAttribute('defer-hydration', '');
            document.body.appendChild(element);
            expect((_b = element.$fastController.mainStyles) === null || _b === void 0 ? void 0 : _b.isAttachedTo(element)).to.be.false;
            document.body.removeChild(element);
        }));
        it("should not invoke a HostBehavior's connectedCallback", () => __awaiter(void 0, void 0, void 0, function* () {
            const behavior = {
                connectedCallback: chai.spy(() => { })
            };
            const { element, controller } = createController();
            element.setAttribute('defer-hydration', '');
            controller.addBehavior(behavior);
            document.body.appendChild(element);
            expect(behavior.connectedCallback).not.to.have.been.called();
            document.body.removeChild(element);
        }));
    });
    describe("when the `defer-hydration` attribute removed after connection", () => {
        it("should render the element's template", () => __awaiter(void 0, void 0, void 0, function* () {
            var _a, _b;
            const { element } = createController({ template: html `<p>Hello world</p>` });
            element.setAttribute('defer-hydration', '');
            document.body.appendChild(element);
            yield Updates.next();
            expect((_a = element.shadowRoot) === null || _a === void 0 ? void 0 : _a.innerHTML).be.equal("");
            element.removeAttribute('defer-hydration');
            yield Updates.next();
            expect((_b = element.shadowRoot) === null || _b === void 0 ? void 0 : _b.innerHTML).to.be.equal("<p>Hello world</p>");
            document.body.removeChild(element);
        }));
        it("should attach the element's main stylesheet", () => __awaiter(void 0, void 0, void 0, function* () {
            var _c, _d;
            const { element } = createController({ styles: css `:host{ color :red}` });
            element.setAttribute('defer-hydration', '');
            document.body.appendChild(element);
            expect((_c = element.$fastController.mainStyles) === null || _c === void 0 ? void 0 : _c.isAttachedTo(element)).to.be.false;
            element.removeAttribute('defer-hydration');
            yield Updates.next();
            expect((_d = element.$fastController.mainStyles) === null || _d === void 0 ? void 0 : _d.isAttachedTo(element)).to.be.true;
            document.body.removeChild(element);
        }));
        it("should invoke a HostBehavior's connectedCallback", () => __awaiter(void 0, void 0, void 0, function* () {
            const behavior = {
                connectedCallback: chai.spy(() => { })
            };
            const { element, controller } = createController();
            element.setAttribute('defer-hydration', '');
            controller.addBehavior(behavior);
            document.body.appendChild(element);
            expect(behavior.connectedCallback).not.to.have.been.called();
            element.removeAttribute('defer-hydration');
            yield Updates.next();
            expect(behavior.connectedCallback).to.have.been.called();
            document.body.removeChild(element);
        }));
    });
});
