var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { expect } from "chai";
import { Observable } from "./observable.js";
import { ArrayObserver, lengthOf } from "./arrays.js";
import { SubscriberSet } from "./notifier.js";
import { Updates } from "./update-queue.js";
describe("The ArrayObserver", () => {
    it("can be retrieved through Observable.getNotifier()", () => {
        ArrayObserver.enable();
        const array = [];
        const notifier = Observable.getNotifier(array);
        expect(notifier).to.be.instanceOf(SubscriberSet);
    });
    it("is the same instance for multiple calls to Observable.getNotifier() on the same array", () => {
        ArrayObserver.enable();
        const array = [];
        const notifier = Observable.getNotifier(array);
        const notifier2 = Observable.getNotifier(array);
        expect(notifier).to.equal(notifier2);
    });
    it("is different for different arrays", () => {
        ArrayObserver.enable();
        const notifier = Observable.getNotifier([]);
        const notifier2 = Observable.getNotifier([]);
        expect(notifier).to.not.equal(notifier2);
    });
    it("doesn't affect for/in loops on arrays when enabled", () => {
        ArrayObserver.enable();
        const array = [1, 2, 3];
        const keys = [];
        for (const key in array) {
            keys.push(key);
        }
        expect(keys).eql(["0", "1", "2"]);
    });
    it("doesn't affect for/in loops on arrays when the array is observed", () => {
        ArrayObserver.enable();
        const array = [1, 2, 3];
        const keys = [];
        const notifier = Observable.getNotifier(array);
        for (const key in array) {
            keys.push(key);
        }
        expect(notifier).to.be.instanceOf(SubscriberSet);
        expect(keys).eql(["0", "1", "2"]);
    });
    it("observes pops", () => __awaiter(void 0, void 0, void 0, function* () {
        ArrayObserver.enable();
        const array = ["foo", "bar", "hello", "world"];
        array.pop();
        expect(array).members(["foo", "bar", "hello"]);
        Array.prototype.pop.call(array);
        expect(array).members(["foo", "bar"]);
        const observer = Observable.getNotifier(array);
        let changeArgs = null;
        observer.subscribe({
            handleChange(array, args) {
                changeArgs = args;
            }
        });
        array.pop();
        expect(array).members(["foo"]);
        yield Updates.next();
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(0);
        expect(changeArgs[0].removed).members(["bar"]);
        expect(changeArgs[0].index).equal(1);
        Array.prototype.pop.call(array);
        expect(array).members([]);
        yield Updates.next();
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(0);
        expect(changeArgs[0].removed).members(["foo"]);
        expect(changeArgs[0].index).equal(0);
    }));
    it("observes pushes", () => __awaiter(void 0, void 0, void 0, function* () {
        ArrayObserver.enable();
        const array = [];
        array.push("foo");
        expect(array).members(["foo"]);
        Array.prototype.push.call(array, "bar");
        expect(array).members(["foo", "bar"]);
        const observer = Observable.getNotifier(array);
        let changeArgs = null;
        observer.subscribe({
            handleChange(array, args) {
                changeArgs = args;
            }
        });
        array.push("hello");
        expect(array).members(["foo", "bar", "hello"]);
        yield Updates.next();
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(1);
        expect(changeArgs[0].removed).members([]);
        expect(changeArgs[0].index).equal(2);
        Array.prototype.push.call(array, "world");
        expect(array).members(["foo", "bar", "hello", "world"]);
        yield Updates.next();
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(1);
        expect(changeArgs[0].removed).members([]);
        expect(changeArgs[0].index).equal(3);
    }));
    it("observes reverses", () => __awaiter(void 0, void 0, void 0, function* () {
        ArrayObserver.enable();
        const array = [1, 2, 3, 4];
        array.reverse();
        expect(array).members([4, 3, 2, 1]);
        Array.prototype.reverse.call(array);
        expect(array).members([1, 2, 3, 4]);
        const observer = Observable.getNotifier(array);
        let changeArgs = null;
        observer.subscribe({
            handleChange(array, args) {
                changeArgs = args;
            }
        });
        array.reverse();
        expect(array).members([4, 3, 2, 1]);
        yield Updates.next();
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(0);
        expect(changeArgs[0].removed).members([]);
        expect(changeArgs[0].index).equal(0);
        expect(changeArgs[0].reset).equal(true);
        Array.prototype.reverse.call(array);
        expect(array).members([1, 2, 3, 4]);
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(0);
        expect(changeArgs[0].removed).members([]);
        expect(changeArgs[0].index).equal(0);
        expect(changeArgs[0].reset).equal(true);
    }));
    it("observes shifts", () => __awaiter(void 0, void 0, void 0, function* () {
        ArrayObserver.enable();
        const array = ["foo", "bar", "hello", "world"];
        array.shift();
        expect(array).members(["bar", "hello", "world"]);
        Array.prototype.shift.call(array);
        expect(array).members(["hello", "world"]);
        const observer = Observable.getNotifier(array);
        let changeArgs = null;
        observer.subscribe({
            handleChange(array, args) {
                changeArgs = args;
            }
        });
        array.shift();
        expect(array).members(["world"]);
        yield Updates.next();
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(0);
        expect(changeArgs[0].removed).members(["hello"]);
        expect(changeArgs[0].index).equal(0);
        Array.prototype.shift.call(array);
        expect(array).members([]);
        yield Updates.next();
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(0);
        expect(changeArgs[0].removed).members(["world"]);
        expect(changeArgs[0].index).equal(0);
    }));
    it("observes sorts", () => __awaiter(void 0, void 0, void 0, function* () {
        ArrayObserver.enable();
        let array = [1, 2, 3, 4];
        array.sort((a, b) => b - a);
        expect(array).members([4, 3, 2, 1]);
        Array.prototype.sort.call(array, (a, b) => a - b);
        expect(array).members([1, 2, 3, 4]);
        const observer = Observable.getNotifier(array);
        let changeArgs = null;
        observer.subscribe({
            handleChange(array, args) {
                changeArgs = args;
            }
        });
        array.sort((a, b) => b - a);
        expect(array).members([4, 3, 2, 1]);
        yield Updates.next();
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(0);
        expect(changeArgs[0].removed).members([]);
        expect(changeArgs[0].index).equal(0);
        expect(changeArgs[0].reset).equal(true);
        Array.prototype.sort.call(array, (a, b) => a - b);
        expect(array).members([1, 2, 3, 4]);
        yield Updates.next();
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(0);
        expect(changeArgs[0].removed).members([]);
        expect(changeArgs[0].index).equal(0);
        expect(changeArgs[0].reset).equal(true);
    }));
    it("observes splices", () => __awaiter(void 0, void 0, void 0, function* () {
        ArrayObserver.enable();
        let array = [1, 2, 3, 4];
        array.splice(1, 1, 'hello');
        expect(array).members([1, "hello", 3, 4]);
        Array.prototype.splice.call(array, 2, 1, "world");
        expect(array).members([1, "hello", "world", 4]);
        const observer = Observable.getNotifier(array);
        let changeArgs = null;
        observer.subscribe({
            handleChange(array, args) {
                changeArgs = args;
            }
        });
        array.splice(1, 1, "foo");
        expect(array).members([1, "foo", "world", 4]);
        yield Updates.next();
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(1);
        expect(changeArgs[0].removed).members(["hello"]);
        expect(changeArgs[0].index).equal(1);
        Array.prototype.splice.call(array, 2, 1, 'bar');
        expect(array).members([1, "foo", "bar", 4]);
        yield Updates.next();
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(1);
        expect(changeArgs[0].removed).members(["world"]);
        expect(changeArgs[0].index).equal(2);
    }));
    it("observes unshifts", () => __awaiter(void 0, void 0, void 0, function* () {
        ArrayObserver.enable();
        let array = [];
        array.unshift("foo");
        expect(array).members(["foo"]);
        Array.prototype.unshift.call(array, "bar");
        expect(array).members(["bar", "foo"]);
        const observer = Observable.getNotifier(array);
        let changeArgs = null;
        observer.subscribe({
            handleChange(array, args) {
                changeArgs = args;
            }
        });
        array.unshift("hello");
        expect(array).members(["hello", "bar", "foo"]);
        yield Updates.next();
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(1);
        expect(changeArgs[0].removed).members([]);
        expect(changeArgs[0].index).equal(0);
        Array.prototype.unshift.call(array, 'world');
        expect(array).members(["world", "hello", "bar", "foo"]);
        yield Updates.next();
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(1);
        expect(changeArgs[0].removed).members([]);
        expect(changeArgs[0].index).equal(0);
    }));
    it("observes back to back array modification operations", () => __awaiter(void 0, void 0, void 0, function* () {
        ArrayObserver.enable();
        let array = [];
        array.unshift("foo");
        expect(array).members(["foo"]);
        Array.prototype.unshift.call(array, "bar");
        expect(array).members(["bar", "foo"]);
        const observer = Observable.getNotifier(array);
        let changeArgs = null;
        observer.subscribe({
            handleChange(array, args) {
                changeArgs = args;
            }
        });
        array.unshift("hello");
        expect(array).members(["hello", "bar", "foo"]);
        yield Updates.next();
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(1);
        expect(changeArgs[0].removed).members([]);
        expect(changeArgs[0].index).equal(0);
        Array.prototype.shift.call(array);
        expect(array).members(["bar", "foo"]);
        yield Updates.next();
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(0);
        expect(changeArgs[0].removed).members(['hello']);
        expect(changeArgs[0].index).equal(0);
        array.unshift("hello", "world");
        expect(array).members(["hello", "world", "bar", "foo"]);
        yield Updates.next();
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(2);
        expect(changeArgs[0].removed).members([]);
        expect(changeArgs[0].index).equal(0);
        Array.prototype.unshift.call(array, "hi", "there");
        expect(array).members(["hi", "there", "hello", "world", "bar", "foo"]);
        yield Updates.next();
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(2);
        expect(changeArgs[0].removed).members([]);
        expect(changeArgs[0].index).equal(0);
        Array.prototype.splice.call(array, 2, 2, "bye", "foo");
        expect(array).members(["hi", "there", "bye", "foo", "bar", "foo"]);
        yield Updates.next();
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(2);
        expect(changeArgs[0].removed).members(["hello", "world"]);
        expect(changeArgs[0].index).equal(2);
        Array.prototype.splice.call(array, 1, 0, "hello");
        expect(array).members(["hi", "there", "hello", "bye", "foo", "bar", "foo"]);
        yield Updates.next();
        expect(changeArgs).length(1);
        expect(changeArgs[0].addedCount).equal(1);
        expect(changeArgs[0].removed).members([]);
        expect(changeArgs[0].index).equal(1);
    }));
    it("should not deliver splices for changes prior to subscription", () => __awaiter(void 0, void 0, void 0, function* () {
        ArrayObserver.enable();
        const array = [1, 2, 3, 4, 5];
        const observer = Observable.getNotifier(array);
        let wasCalled = false;
        array.push(6);
        observer.subscribe({
            handleChange() {
                wasCalled = true;
            }
        });
        yield Updates.next();
        expect(wasCalled).to.be.false;
    }));
    it("should not deliver splices for .splice() when .splice() does not change the items in the array", () => __awaiter(void 0, void 0, void 0, function* () {
        ArrayObserver.enable();
        const array = [1, 2, 3, 4, 5];
        const observer = Observable.getNotifier(array);
        let splices;
        observer.subscribe({
            handleChange(source, args) {
                splices = args;
            }
        });
        array.splice(0, array.length, ...array);
        yield Updates.next();
        expect(splices.length).to.equal(0);
    }));
});
describe("The array length observer", () => {
    class Model {
    }
    it("returns zero length if the array is undefined", () => __awaiter(void 0, void 0, void 0, function* () {
        const instance = new Model();
        const observer = Observable.binding(x => lengthOf(x.items));
        const value = observer.observe(instance);
        expect(value).to.equal(0);
        observer.dispose();
    }));
    it("returns zero length if the array is null", () => __awaiter(void 0, void 0, void 0, function* () {
        const instance = new Model();
        instance.items = null;
        const observer = Observable.binding(x => lengthOf(x.items));
        const value = observer.observe(instance);
        expect(value).to.equal(0);
        observer.dispose();
    }));
    it("returns length of an array", () => __awaiter(void 0, void 0, void 0, function* () {
        const instance = new Model();
        instance.items = [1, 2, 3, 4, 5];
        const observer = Observable.binding(x => lengthOf(x.items));
        const value = observer.observe(instance);
        expect(value).to.equal(5);
        observer.dispose();
    }));
    it("notifies when the array length changes", () => __awaiter(void 0, void 0, void 0, function* () {
        const instance = new Model();
        instance.items = [1, 2, 3, 4, 5];
        let changed = false;
        const observer = Observable.binding(x => lengthOf(x.items), {
            handleChange() {
                changed = true;
            }
        });
        const value = observer.observe(instance);
        expect(value).to.equal(5);
        instance.items.push(6);
        yield Updates.next();
        expect(changed).to.be.true;
        expect(observer.observe(instance)).to.equal(6);
        observer.dispose();
    }));
    it("does not notify on changes that don't change the length", () => __awaiter(void 0, void 0, void 0, function* () {
        const instance = new Model();
        instance.items = [1, 2, 3, 4, 5];
        let changed = false;
        const observer = Observable.binding(x => lengthOf(x.items), {
            handleChange() {
                changed = true;
            }
        });
        const value = observer.observe(instance);
        expect(value).to.equal(5);
        instance.items.splice(2, 1, 6);
        yield Updates.next();
        expect(changed).to.be.false;
        expect(observer.observe(instance)).to.equal(5);
        observer.dispose();
    }));
});
