var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { expect } from "chai";
import { AdoptedStyleSheetsStrategy, StyleElementStrategy } from "../components/element-controller.js";
import { uniqueElementName } from "../testing/fixture.js";
import { cssDirective } from "./css-directive.js";
import { css } from "./css.js";
import { ElementStyles } from "./element-styles.js";
import { html } from "../templating/template.js";
import { ref } from "../templating/ref.js";
import { FASTElement, customElement } from "../components/fast-element.js";
import { ExecutionContext } from "../observation/observable.js";
import { CSSBindingDirective } from "./css-binding-directive.js";
import { Binding } from "../binding/binding.js";
import { oneTime } from "../binding/one-time.js";
if (ElementStyles.supportsAdoptedStyleSheets) {
    describe("AdoptedStyleSheetsStrategy", () => {
        context("when adding and removing styles", () => {
            it("should remove an associated stylesheet", () => {
                const strategy = new AdoptedStyleSheetsStrategy([``]);
                const target = {
                    adoptedStyleSheets: [],
                };
                strategy.addStylesTo(target);
                expect(target.adoptedStyleSheets.length).to.equal(1);
                strategy.removeStylesFrom(target);
                expect(target.adoptedStyleSheets.length).to.equal(0);
            });
            it("should not remove unassociated styles", () => {
                const strategy = new AdoptedStyleSheetsStrategy(["test"]);
                const style = new CSSStyleSheet();
                const target = {
                    adoptedStyleSheets: [style],
                };
                strategy.addStylesTo(target);
                expect(target.adoptedStyleSheets.length).to.equal(2);
                expect(target.adoptedStyleSheets).to.contain(strategy.sheets[0]);
                strategy.removeStylesFrom(target);
                expect(target.adoptedStyleSheets.length).to.equal(1);
                expect(target.adoptedStyleSheets).not.to.contain(strategy.sheets[0]);
            });
            it("should track when added and removed from a target", () => {
                const styles = ``;
                const elementStyles = new ElementStyles([styles]);
                const target = {
                    adoptedStyleSheets: [],
                };
                expect(elementStyles.isAttachedTo(target)).to.equal(false);
                elementStyles.addStylesTo(target);
                expect(elementStyles.isAttachedTo(target)).to.equal(true);
                elementStyles.removeStylesFrom(target);
                expect(elementStyles.isAttachedTo(target)).to.equal(false);
            });
            it("should order HTMLStyleElement order by addStyleTo() call order", () => {
                const red = new AdoptedStyleSheetsStrategy(['r']);
                const green = new AdoptedStyleSheetsStrategy(['g']);
                const target = {
                    adoptedStyleSheets: [],
                };
                red.addStylesTo(target);
                green.addStylesTo(target);
                expect((target.adoptedStyleSheets[0])).to.equal(red.sheets[0]);
                expect((target.adoptedStyleSheets[1])).to.equal(green.sheets[0]);
            });
            it("should order HTMLStyleElements in array order of provided sheets", () => {
                const red = new AdoptedStyleSheetsStrategy(['r', 'g']);
                const target = {
                    adoptedStyleSheets: [],
                };
                red.addStylesTo(target);
                expect((target.adoptedStyleSheets[0])).to.equal(red.sheets[0]);
                expect((target.adoptedStyleSheets[1])).to.equal(red.sheets[1]);
            });
            it("should apply stylesheets to the shadowRoot of a provided element when the shadowRoot is publicly accessible", () => {
                const strategy = new AdoptedStyleSheetsStrategy([``]);
                const target = {
                    shadowRoot: {
                        adoptedStyleSheets: [],
                    }
                };
                strategy.addStylesTo(target);
                expect(target.shadowRoot.adoptedStyleSheets.length).to.equal(1);
                strategy.removeStylesFrom(target);
                expect(target.shadowRoot.adoptedStyleSheets.length).to.equal(0);
            });
            it("should apply stylesheets to the shadowRoot of a provided FASTElement when defined with a closed shadowRoot", () => {
                const name = uniqueElementName();
                let MyElement = class MyElement extends FASTElement {
                    get styleTarget() {
                        return this.pChild.getRootNode();
                    }
                };
                MyElement = __decorate([
                    customElement({
                        name,
                        template: html `<p ${ref("pChild")}></p>`,
                        shadowOptions: {
                            mode: "closed"
                        }
                    })
                ], MyElement);
                const strategy = new AdoptedStyleSheetsStrategy([``]);
                const target = document.createElement(name);
                document.body.appendChild(target);
                strategy.addStylesTo(target);
                expect(target.styleTarget.adoptedStyleSheets.length).to.equal(1);
                strategy.removeStylesFrom(target);
                expect(target.styleTarget.adoptedStyleSheets.length).to.equal(0);
                document.body.removeChild(target);
            });
            it("should apply stylesheets to the parent document of the provided element when the shadowRoot of the element is inaccessible or doesn't exist and the element is in light DOM", () => {
                const target = document.createElement("div");
                target.attachShadow({ mode: "closed" });
                document.body.appendChild(target);
                const strategy = new AdoptedStyleSheetsStrategy([``]);
                strategy.addStylesTo(target);
                expect(document.adoptedStyleSheets.length).to.equal(1);
                strategy.removeStylesFrom(target);
                expect(document.adoptedStyleSheets.length).to.equal(0);
                document.body.removeChild(target);
            });
            it("should apply stylesheets to the host's shadowRoot when the shadowRoot of the element is inaccessible or doesn't exist and the element is in a shadowRoot", () => {
                const strategy = new AdoptedStyleSheetsStrategy([``]);
                const host = document.createElement('div');
                const target = document.createElement('div');
                const hostShadow = host.attachShadow({ mode: "closed" });
                target.attachShadow({ mode: "closed" });
                hostShadow.appendChild(target);
                document.body.appendChild(host);
                strategy.addStylesTo(target);
                expect(hostShadow.adoptedStyleSheets.length).to.equal(1);
                strategy.removeStylesFrom(target);
                expect(hostShadow.adoptedStyleSheets.length).to.equal(0);
                document.body.removeChild(host);
            });
        });
    });
}
describe("StyleElementStrategy", () => {
    it("can add and remove from the document directly", () => {
        const styles = [``];
        const elementStyles = new ElementStyles(styles)
            .withStrategy(StyleElementStrategy);
        document.body.innerHTML = "";
        elementStyles.addStylesTo(document);
        expect(document.body.childNodes[0]).to.be.instanceof(HTMLStyleElement);
        elementStyles.removeStylesFrom(document);
        expect(document.body.childNodes.length).to.equal(0);
    });
    it("can add and remove from a ShadowRoot", () => {
        const styles = ``;
        const strategy = new StyleElementStrategy([styles]);
        document.body.innerHTML = "";
        const element = document.createElement("div");
        const shadowRoot = element.attachShadow({ mode: "open" });
        strategy.addStylesTo(shadowRoot);
        expect(shadowRoot.childNodes[0]).to.be.instanceof(HTMLStyleElement);
        strategy.removeStylesFrom(shadowRoot);
        expect(shadowRoot.childNodes.length).to.equal(0);
    });
    it("should track when added and removed from a target", () => {
        const styles = ``;
        const elementStyles = new ElementStyles([styles]);
        document.body.innerHTML = "";
        expect(elementStyles.isAttachedTo(document)).to.equal(false);
        elementStyles.addStylesTo(document);
        expect(elementStyles.isAttachedTo(document)).to.equal(true);
        elementStyles.removeStylesFrom(document);
        expect(elementStyles.isAttachedTo(document)).to.equal(false);
    });
    it("should order HTMLStyleElement order by addStyleTo() call order", () => {
        const red = new StyleElementStrategy([`body:{color:red;}`]);
        const green = new StyleElementStrategy([`body:{color:green;}`]);
        document.body.innerHTML = "";
        const element = document.createElement("div");
        const shadowRoot = element.attachShadow({ mode: "open" });
        red.addStylesTo(shadowRoot);
        green.addStylesTo(shadowRoot);
        expect(shadowRoot.childNodes[0].innerHTML).to.equal("body:{color:red;}");
        expect(shadowRoot.childNodes[1].innerHTML).to.equal("body:{color:green;}");
    });
    it("should order the HTMLStyleElements in array order of provided sheets", () => {
        const red = new StyleElementStrategy([`body:{color:red;}`, `body:{color:green;}`]);
        document.body.innerHTML = "";
        const element = document.createElement("div");
        const shadowRoot = element.attachShadow({ mode: "open" });
        red.addStylesTo(shadowRoot);
        expect(shadowRoot.childNodes[0].innerHTML).to.equal("body:{color:red;}");
        expect(shadowRoot.childNodes[1].innerHTML).to.equal("body:{color:green;}");
    });
    it("should apply stylesheets to the shadowRoot of a provided element when the shadowRoot is publicly accessible", () => {
        const css = ":host{color:red}";
        const strategy = new StyleElementStrategy([css]);
        const element = document.createElement("div");
        const shadowRoot = element.attachShadow({ mode: "open" });
        strategy.addStylesTo(shadowRoot);
        expect(shadowRoot.childNodes[0].innerHTML).to.equal(css);
        strategy.removeStylesFrom(shadowRoot);
        expect(shadowRoot.childNodes[0]).to.equal(undefined);
    });
    it("should apply stylesheets to the shadowRoot of a provided FASTElement when defined with a closed shadowRoot", () => {
        const css = ":host{color:red}";
        const name = uniqueElementName();
        let MyElement = class MyElement extends FASTElement {
            get styleTarget() {
                return this.pChild.getRootNode();
            }
        };
        MyElement = __decorate([
            customElement({
                name,
                template: html `<p ${ref("pChild")}></p>`,
                shadowOptions: {
                    mode: "closed"
                }
            })
        ], MyElement);
        const strategy = new StyleElementStrategy([css]);
        const target = document.createElement(name);
        document.body.appendChild(target);
        strategy.addStylesTo(target);
        expect(target.styleTarget.childNodes[2].innerHTML).to.equal(css);
        strategy.removeStylesFrom(target);
        expect(target.styleTarget.childNodes[2]).to.equal(undefined);
        document.body.removeChild(target);
    });
    it("should apply stylesheets to the parent document of the provided element when the shadowRoot of the element is inaccessible or doesn't exist and the element is in light DOM", () => {
        const target = document.createElement("div");
        const css = ":host{color:red}";
        target.attachShadow({ mode: "closed" });
        document.body.appendChild(target);
        const strategy = new StyleElementStrategy([css]);
        strategy.addStylesTo(target);
        expect(document.body.childNodes[1].innerHTML).to.equal(css);
        strategy.removeStylesFrom(target);
        expect(document.body.childNodes[1]).to.equal(undefined);
        document.body.removeChild(target);
    });
    it("should apply stylesheets to the host's shadowRoot when the shadowRoot of the element is inaccessible or doesn't exist and the element is in a shadowRoot", () => {
        const css = ":host{color:red}";
        const strategy = new StyleElementStrategy([css]);
        const host = document.createElement('div');
        const target = document.createElement('div');
        const hostShadow = host.attachShadow({ mode: "closed" });
        target.attachShadow({ mode: "closed" });
        hostShadow.appendChild(target);
        document.body.appendChild(host);
        strategy.addStylesTo(target);
        expect(hostShadow.childNodes[1].innerHTML).to.equal(css);
        strategy.removeStylesFrom(target);
        expect(hostShadow.adoptedStyleSheets.length).to.equal(0);
        expect(hostShadow.childNodes[1]).to.equal(undefined);
        document.body.removeChild(host);
    });
});
describe("ElementStyles", () => {
    it("can create from a string", () => {
        const css = ".class { color: red; }";
        const styles = new ElementStyles([css]);
        expect(styles.styles).to.contain(css);
    });
    it("can create from multiple strings", () => {
        const css1 = ".class { color: red; }";
        const css2 = ".class2 { color: red; }";
        const styles = new ElementStyles([css1, css2]);
        expect(styles.styles).to.contain(css1);
        expect(styles.styles.indexOf(css1)).to.equal(0);
        expect(styles.styles).to.contain(css2);
    });
    it("can create from an ElementStyles", () => {
        const css = ".class { color: red; }";
        const existingStyles = new ElementStyles([css]);
        const styles = new ElementStyles([existingStyles]);
        expect(styles.styles).to.contain(existingStyles);
    });
    it("can create from multiple ElementStyles", () => {
        const css1 = ".class { color: red; }";
        const css2 = ".class2 { color: red; }";
        const existingStyles1 = new ElementStyles([css1]);
        const existingStyles2 = new ElementStyles([css2]);
        const styles = new ElementStyles([existingStyles1, existingStyles2]);
        expect(styles.styles).to.contain(existingStyles1);
        expect(styles.styles.indexOf(existingStyles1)).to.equal(0);
        expect(styles.styles).to.contain(existingStyles2);
    });
    it("can create from mixed strings and ElementStyles", () => {
        const css1 = ".class { color: red; }";
        const css2 = ".class2 { color: red; }";
        const existingStyles2 = new ElementStyles([css2]);
        const styles = new ElementStyles([css1, existingStyles2]);
        expect(styles.styles).to.contain(css1);
        expect(styles.styles.indexOf(css1)).to.equal(0);
        expect(styles.styles).to.contain(existingStyles2);
    });
    if (ElementStyles.supportsAdoptedStyleSheets) {
        it("can create from a CSSStyleSheet", () => {
            const styleSheet = new CSSStyleSheet();
            const styles = new ElementStyles([styleSheet]);
            expect(styles.styles).to.contain(styleSheet);
        });
        it("can create from multiple CSSStyleSheets", () => {
            const styleSheet1 = new CSSStyleSheet();
            const styleSheet2 = new CSSStyleSheet();
            const styles = new ElementStyles([styleSheet1, styleSheet2]);
            expect(styles.styles).to.contain(styleSheet1);
            expect(styles.styles.indexOf(styleSheet1)).to.equal(0);
            expect(styles.styles).to.contain(styleSheet2);
        });
        it("can create from mixed strings, ElementStyles, and CSSStyleSheets", () => {
            const css1 = ".class { color: red; }";
            const css2 = ".class2 { color: red; }";
            const existingStyles2 = new ElementStyles([css2]);
            const styleSheet3 = new CSSStyleSheet();
            const styles = new ElementStyles([css1, existingStyles2, styleSheet3]);
            expect(styles.styles).to.contain(css1);
            expect(styles.styles.indexOf(css1)).to.equal(0);
            expect(styles.styles).to.contain(existingStyles2);
            expect(styles.styles.indexOf(existingStyles2)).to.equal(1);
            expect(styles.styles).to.contain(styleSheet3);
        });
    }
});
describe("css", () => {
    describe("with a CSSDirective", () => {
        describe("should interpolate the product of CSSDirective.createCSS() into the resulting ElementStyles CSS", () => {
            it("when the result is a string", () => {
                let Directive = class Directive {
                    createCSS() {
                        return "red";
                    }
                };
                Directive = __decorate([
                    cssDirective()
                ], Directive);
                const styles = css `host: {color: ${new Directive()};}`;
                expect(styles.styles.some(x => x === "host: {color: red;}")).to.equal(true);
            });
            it("when the result is an ElementStyles", () => {
                const _styles = css `:host{color: red}`;
                let Directive = class Directive {
                    createCSS() {
                        return _styles;
                    }
                };
                Directive = __decorate([
                    cssDirective()
                ], Directive);
                const styles = css `${new Directive()}`;
                expect(styles.styles.includes(_styles)).to.equal(true);
            });
            if (ElementStyles.supportsAdoptedStyleSheets) {
                it("when the result is a CSSStyleSheet", () => {
                    const _styles = new CSSStyleSheet();
                    let Directive = class Directive {
                        createCSS() {
                            return _styles;
                        }
                    };
                    Directive = __decorate([
                        cssDirective()
                    ], Directive);
                    const styles = css `${new Directive()}`;
                    expect(styles.styles.includes(_styles)).to.equal(true);
                });
            }
        });
        it("should add the behavior returned from CSSDirective.getBehavior() to the resulting ElementStyles", () => {
            var _a;
            const behavior = {
                addedCallback() { },
            };
            let Directive = class Directive {
                createCSS(add) {
                    add(behavior);
                    return "";
                }
            };
            Directive = __decorate([
                cssDirective()
            ], Directive);
            const styles = css `${new Directive()}`;
            expect((_a = styles.behaviors) === null || _a === void 0 ? void 0 : _a.includes(behavior)).to.equal(true);
        });
    });
    describe("bindings", () => {
        class Model {
            constructor(color) {
                this.color = color;
            }
        }
        ;
        it("can be created from interpolated functions", () => {
            const styles = css `host: { color: ${x => x.color}; }`;
            const bindings = styles.behaviors.filter(x => x instanceof CSSBindingDirective);
            expect(bindings.length).equals(1);
            const b = bindings[0];
            expect(b.dataBinding).instanceof(Binding);
            expect(b.targetAspect.startsWith("--v")).true;
            const result = b.dataBinding.evaluate(new Model("red"), ExecutionContext.default);
            expect(result).equals("red");
            expect(styles.styles[0].indexOf("var(--")).not.equal(-1);
        });
        it("can be created from interpolated bindings", () => {
            const styles = css `host: { color: ${oneTime(x => x.color)}; }`;
            const bindings = styles.behaviors.filter(x => x instanceof CSSBindingDirective);
            expect(bindings.length).equals(1);
            const b = bindings[0];
            expect(b.dataBinding).instanceof(Binding);
            expect(b.targetAspect.startsWith("--v")).true;
            const result = b.dataBinding.evaluate(new Model("red"), ExecutionContext.default);
            expect(result).equals("red");
            expect(styles.styles[0].indexOf("var(--")).not.equal(-1);
        });
    });
});
describe("cssPartial", () => {
    it("should have a createCSS method that is the CSS string interpolated with the createCSS product of any CSSDirectives", () => {
        const add = () => void 0;
        let myDirective = class myDirective {
            createCSS() { return "red"; }
            ;
        };
        myDirective = __decorate([
            cssDirective()
        ], myDirective);
        const partial = css.partial `color: ${new myDirective}`;
        expect(partial.createCSS(add)).to.equal("color: red");
    });
    it("Should add behaviors from interpolated CSS directives", () => {
        const behavior = {
            addedCallback() { },
        };
        const behavior2 = Object.assign({}, behavior);
        let directive = class directive {
            createCSS(add) {
                add(behavior);
                return "";
            }
            ;
        };
        directive = __decorate([
            cssDirective()
        ], directive);
        let directive2 = class directive2 {
            createCSS(add) {
                add(behavior2);
                return "";
            }
            ;
        };
        directive2 = __decorate([
            cssDirective()
        ], directive2);
        const partial = css.partial `${new directive}${new directive2}`;
        const behaviors = [];
        const add = (x) => behaviors.push(x);
        partial.createCSS(add);
        expect(behaviors[0]).to.equal(behavior);
        expect(behaviors[1]).to.equal(behavior2);
    });
    it("should add any ElementStyles interpolated into the template function when bound to an element", () => {
        const styles = css `:host {color: blue; }`;
        const partial = css.partial `${styles}`;
        const capturedBehaviors = [];
        let addStylesCalled = false;
        const controller = {
            mainStyles: null,
            isConnected: false,
            isBound: false,
            source: {},
            context: ExecutionContext.default,
            addStyles(style) {
                expect(style.styles.includes(styles)).to.be.true;
                addStylesCalled = true;
            },
            removeStyles(styles) { },
            addBehavior() { },
            removeBehavior() { },
            onUnbind() { }
        };
        const add = (x) => capturedBehaviors.push(x);
        partial.createCSS(add);
        expect(capturedBehaviors[0]).to.equal(partial);
        partial.addedCallback(controller);
        expect(addStylesCalled).to.be.true;
    });
});
